{***************************************************************************}
{  Date: 19.01.97                                            Time: 17:30:40 }
{  Copyright (c)1996-1997                                                   }
{   by Dr.Plass                                               TFindProc1.0 }
{                                                                           }
{                                              for DELPHI 2, Win95 (Not NT!)}
{  feel free to contact me:                                                 }
{  Peter.Plass@FH-Zwickau.de                                                }
{  http://www.fh-zwickau.de/~pp/tm.htm                                      }
{                                                                           }
{  All Rights Reserved.                                                     }
{  This component can be freely used and distributed in commercial and      }
{  private environments, provided this notice is not modified in any way.   }
{                                                                           }
{                                                                           }
{***************************************************************************}
//
//
//  Methods:
//
//   procedure GetProcessInfo;
//    function GetP_CountUsage(idx : Integer):DWORD;
//    function GetP_ID(idx : Integer): DWORD;
//    function GetP_defHeap(idx : Integer): DWORD;
//    function GetP_ModuleID(idx : Integer): DWORD;
//    function GetP_CountThreads(idx : Integer): DWORD;
//    function GetP_ParentPID(idx : Integer): DWORD;
//    function GetP_Priority(idx : Integer): LongInt;
//    function GetP_dwFlags(idx : Integer): DWORD;
//    function GetP_ExePath(idx : Integer): String;

//  procedure GetModuleInfo(ProcID : DWORD);
//    function GetM_ModuleID(idx : Integer):DWORD;
//    function GetM_ProcessID(idx : Integer):DWORD;
//    function GetM_GlblcntUsage(idx : Integer):DWORD;
//    function GetM_ProccntUsage(idx : Integer):DWORD;
//    function GetM_modBaseSize(idx : Integer):DWORD;
//    function GetM_hModule(idx : Integer):HMODULE;
//    function GetM_szExePath(idx : Integer):string;

//  e.g.: whdl := ProcList1.GetP_CountUsage(ListBox1.ItemIndex);
//
//  Properties:
//
//  ModuleList : TStringList ;
//  ProcessList: TStringList ;
//


unit FindProc;
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, toolhelp32;

const
 cAbout = 'TFindProc1.0:  1997 by Dr.Plass';

type
 TProcessInfo = class(TObject)
  private
  public
   cntUsage,
   ProcessID,    	   // this process
   DefaultHeapID,
   ModuleID,     	   // associated exe
   cntThreads,
   ParentProcessID: DWORD; // this process's parent process
   PriClassBase: LongInt;  // Base priority of process's threads
   dwFlags: DWORD;
   ExePath : String;
  end;

 TModuleInfo = class(TObject)
  private
  public
   ModuleID,	        // This module
   ProcessID,	        // owning process
   GlblcntUsage,	// Global usage count on the module
   ProccntUsage: DWORD;	// Module usage count in th32ProcessID's context
   modBaseAddr: PBYTE; 	// Base address of module in th32ProcessID's context
   modBaseSize: DWORD;  // Size in bytes of module starting at modBaseAddr
   hModule: HMODULE;  	// The hModule of this module in th32ProcessID's context
   szExePath: string;
  end;

 TProcList = class(TComponent)
  private
   MObj              : TModuleInfo;
   PObj              : TProcessInfo;
   fProcessList,
   fModuleList       : TStringList;
   fAbout            : String;
   fPID              : DWORD;

  protected
   procedure FuAbout(value :String);
  public
   Constructor Create(AOwner : TComponent); override;
   Destructor Destroy; override;

   procedure GetProcessInfo;
    function GetP_CountUsage(idx : Integer):DWORD;
    function GetP_ID(idx : Integer): DWORD;
    function GetP_defHeap(idx : Integer): DWORD;
    function GetP_ModuleID(idx : Integer): DWORD;
    function GetP_CountThreads(idx : Integer): DWORD;
    function GetP_ParentPID(idx : Integer): DWORD;
    function GetP_Priority(idx : Integer): LongInt;
    function GetP_dwFlags(idx : Integer): DWORD;
    function GetP_ExePath(idx : Integer): String;

   procedure GetModuleInfo(ProcID : DWORD);
    function GetM_ModuleID(idx : Integer):DWORD;
    function GetM_ProcessID(idx : Integer):DWORD;
    function GetM_GlblcntUsage(idx : Integer):DWORD;
    function GetM_ProccntUsage(idx : Integer):DWORD;
    function GetM_modBaseSize(idx : Integer):DWORD;
    function GetM_hModule(idx : Integer):HMODULE;
    function GetM_szExePath(idx : Integer):string;

  published
   Property About      : String read FAbout write FuAbout;
   Property ModuleList : TStringList read fModuleList write fModuleList;
   Property ProcessList: TStringList read fProcessList write fProcessList;
  end;

procedure Register;

implementation


Constructor TProcList.Create(AOwner : TComponent);
Begin
 inherited Create(AOwner);
 fModuleList          := TStringList.Create;
 fModuleList.sorted   := True;
 fProcessList         := TStringList.Create;
 fProcessList.sorted  := True;
 fAbout               := '';
End;

Destructor TProcList.Destroy;
Begin
 fModuleList.Clear;
 fModuleList.Free;
 fProcessList.Clear;
 fProcessList.Free;
 MObj.Free;
 MObj := NIL;
 PObj.Free;
 PObj := NIL;
 inherited Destroy;
End;

procedure TProcList.GetModuleInfo(ProcID : DWORD);
var
 snap   : THandle;
 me32   : TMODULEENTRY32;
begin
 snap := 0;
 fModuleList.Clear;

 try
  snap := CreateToolhelp32Snapshot(TH32CS_SNAPMODULE,
                                   ProcID);
  if snap <> 0 then begin
   me32.dwSize := SizeOf(TMODULEENTRY32);
   if Module32First(snap, me32) then begin
    if me32.th32ProcessID = ProcID then
     MObj  := TModuleInfo.Create;
     MObj.ModuleId := me32.th32ModuleID;
     MObj.ProcessID:= me32.th32ProcessID;
     MObj.GlblcntUsage:= me32.GlblcntUsage;
     MObj.ProccntUsage:= me32.ProccntUsage;
     MObj.modBaseAddr:= me32.modBaseAddr;
     MObj.modBaseSize:= me32.modBaseSize;
     Mobj.hModule := me32.hModule;
     MObj.szExePath := String(me32.szExePath);
     fModuleList.AddObject(String(me32.szModule), MObj);
    while Module32Next(snap, me32) do begin
     if me32.th32ProcessID = ProcID then begin
      MObj  := TModuleInfo.Create;
      MObj.ModuleId := me32.th32ModuleID;
      MObj.ProcessID:= me32.th32ProcessID;
      MObj.GlblcntUsage:= me32.GlblcntUsage;
      MObj.ProccntUsage:= me32.ProccntUsage;
      MObj.modBaseAddr:= me32.modBaseAddr;
      MObj.modBaseSize:= me32.modBaseSize;
      Mobj.hModule := me32.hModule;
      MObj.szExePath := String(me32.szExePath);
      fModuleList.AddObject(String(me32.szModule), MObj);
     end;
    end;
   end;
  end;
 finally
  CloseHandle(snap);
 end;
end;

procedure TProcList.GetProcessInfo;
var
 snap   : THandle;
 pe32   : TPROCESSENTRY32;
begin
 snap := 0;
 fProcessList.Clear;

 try
  snap := CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);

  if snap <> 0 then begin
   pe32.dwSize := SizeOf(TPROCESSENTRY32);
   if Process32First(snap, pe32) then begin
    PObj := TProcessInfo.Create;
    PObj.cntUsage       := pe32.cntUsage;
    PObj.ProcessID      := pe32.th32ProcessID;
    PObj.DefaultHeapID  := pe32.th32DefaultHeapID;
    PObj.ModuleID       := pe32.th32ModuleID;
    PObj.cntThreads     := pe32.cntThreads;
    PObj.ParentProcessID:= pe32.th32ParentProcessID;
    PObj.PriClassBase   := pe32.pcPriClassBase;
    Pobj.dwFlags        := pe32.dwFlags;
    PObj.ExePath        := String(pe32.szExeFile);
    fProcessList.AddObject(ExtractFileName(pe32.szExeFile), PObj);
    while Process32Next(snap, pe32) do begin
     PObj  := TProcessInfo.Create;
     PObj.cntUsage       := pe32.cntUsage;
     PObj.ProcessID      := pe32.th32ProcessID;
     PObj.DefaultHeapID  := pe32.th32DefaultHeapID;
     PObj.ModuleID       := pe32.th32ModuleID;
     PObj.cntThreads     := pe32.cntThreads;
     PObj.ParentProcessID:= pe32.th32ParentProcessID;
     PObj.PriClassBase   := pe32.pcPriClassBase;
     Pobj.dwFlags        := pe32.dwFlags;
     PObj.ExePath        := String(pe32.szExeFile);
     fProcessList.AddObject(ExtractFileName(pe32.szExeFile), PObj);
    end;
   end;
  end;

 finally
  CloseHandle(snap);
 end;
End;

procedure TProcList.FuAbout(value:String);
begin
 if value <> fAbout then begin
  fAbout := cAbout;
  MessageDlg('Komponente: TFindProc1.0            '+ #10#13+ #10#13+
             '1997 by Dr.Plass                   '+ #10#13+
             'Peter.Plass@fh-zwickau.de           '+ #10#13+
             'http://www.fh-zwickau.de/~pp/tm.htm ',
              mtInformation, [mbOk],0);
  end;
end;


function TProcList.GetP_CountUsage(idx : Integer):DWORD;
begin
 Result := TProcessInfo(fProcessList.Objects[idx]).cntUsage;
end;
function TProcList.GetP_ID(idx : Integer): DWORD;
begin
 Result := TProcessInfo(fProcessList.Objects[idx]).ProcessID;
end;
function TProcList.GetP_defHeap(idx : Integer): DWORD;
begin
 Result := TProcessInfo(fProcessList.Objects[idx]).DefaultHeapID;
end;
function TProcList.GetP_ModuleID(idx : Integer): DWORD;
begin
 Result := TProcessInfo(fProcessList.Objects[idx]).ModuleID;
end;
function TProcList.GetP_CountThreads(idx : Integer): DWORD;
begin
 Result := TProcessInfo(fProcessList.Objects[idx]).cntThreads;
end;
function TProcList.GetP_ParentPID(idx : Integer): DWORD;
begin
 Result := TProcessInfo(fProcessList.Objects[idx]).ParentProcessID;
end;
function TProcList.GetP_Priority(idx : Integer): LongInt;
begin
 Result := TProcessInfo(fProcessList.Objects[idx]).PriClassBase;
end;
function TProcList.GetP_dwFlags(idx : Integer): DWORD;
begin
 Result := TProcessInfo(fProcessList.Objects[idx]).dwFlags;
end;
function TProcList.GetP_ExePath(idx : Integer): String;
begin
 Result := TProcessInfo(fProcessList.Objects[idx]).ExePath;
end;

function TProcList.GetM_ModuleID(idx : Integer):DWORD;
begin
 Result := TModuleInfo(fModuleList.Objects[idx]).ModuleID;
end;
function TProcList.GetM_ProcessID(idx : Integer):DWORD;
begin
 Result := TModuleInfo(fModuleList.Objects[idx]).ProcessID;
end;
function TProcList.GetM_GlblcntUsage(idx : Integer):DWORD;
begin
 Result := TModuleInfo(fModuleList.Objects[idx]).GlblcntUsage;
end;
function TProcList.GetM_ProccntUsage(idx : Integer):DWORD;
begin
 Result := TModuleInfo(fModuleList.Objects[idx]).ProccntUsage;
end;
function TProcList.GetM_modBaseSize(idx : Integer):DWORD;
begin
 Result := TModuleInfo(fModuleList.Objects[idx]).modBaseSize;
end;
function TProcList.GetM_hModule(idx : Integer):HMODULE;
begin
 Result := TModuleInfo(fModuleList.Objects[idx]).hModule;
end;
function TProcList.GetM_szExePath(idx : Integer):string;
begin
 Result := TModuleInfo(fModuleList.Objects[idx]).szExePath;
end;


procedure Register;
begin
  RegisterComponents('Plass', [TProcList]);
end;
end.







